==> vision-camera-plugin-scan-faces-0.2.4.tgz: package/android/src/main/java/com/visioncamerapluginscanfaces/VisionCameraPluginScanFaces.java <==
package com.visioncamerapluginscanfaces;

import static java.lang.Math.ceil;
import android.annotation.SuppressLint;
import android.graphics.PointF;
import android.graphics.Rect;
import android.media.Image;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.WritableNativeArray;
import com.facebook.react.bridge.WritableNativeMap;
import com.google.android.gms.tasks.Task;
import com.google.android.gms.tasks.Tasks;
import com.google.mlkit.vision.common.InputImage;
import com.google.mlkit.vision.face.Face;
import com.google.mlkit.vision.face.FaceContour;
import com.google.mlkit.vision.face.FaceDetection;
import com.google.mlkit.vision.face.FaceDetector;
import com.google.mlkit.vision.face.FaceDetectorOptions;

import com.mrousavy.camera.frameprocessor.Frame;
import com.mrousavy.camera.frameprocessor.FrameProcessorPlugin;
import com.mrousavy.camera.parsers.Orientation;

import org.json.JSONObject;

import java.util.List;
import java.util.Map;
import java.util.ArrayList;


public class VisionCameraPluginScanFaces extends FrameProcessorPlugin {

  FaceDetectorOptions options =
    new FaceDetectorOptions.Builder()
      .setPerformanceMode(FaceDetectorOptions.PERFORMANCE_MODE_FAST)
      .setContourMode(FaceDetectorOptions.CONTOUR_MODE_NONE)
      .setClassificationMode(FaceDetectorOptions.CLASSIFICATION_MODE_ALL)
      .setMinFaceSize(0.15f)
      .build();

  FaceDetector faceDetector = FaceDetection.getClient(options);

  private WritableMap processBoundingBox(Rect boundingBox) {
    WritableMap bounds = Arguments.createMap();

    // Calculate offset (we need to center the overlay on the target)
    Double offsetX =  (boundingBox.exactCenterX() - ceil(boundingBox.width())) / 2.0f;
    Double offsetY =  (boundingBox.exactCenterY() - ceil(boundingBox.height())) / 2.0f;

    Double x = boundingBox.right + offsetX;
    Double y = boundingBox.top + offsetY;



    bounds.putDouble("x", boundingBox.centerX() + (boundingBox.centerX() - x));
    bounds.putDouble("y", boundingBox.centerY() + (y - boundingBox.centerY()));
    bounds.putDouble("width", boundingBox.width());
    bounds.putDouble("height", boundingBox.height());


    bounds.putDouble("boundingCenterX", boundingBox.centerX());
    bounds.putDouble("boundingCenterY", boundingBox.centerY());
    bounds.putDouble("boundingExactCenterX", boundingBox.exactCenterX());
    bounds.putDouble("boundingExactCenterY", boundingBox.exactCenterY());

    return bounds;
  }

  private WritableMap  processFaceContours(Face face) {
    // All faceContours
    int[] faceContoursTypes =
      new int[] {
        FaceContour.FACE,
        FaceContour.LEFT_EYEBROW_TOP,
        FaceContour.LEFT_EYEBROW_BOTTOM,
        FaceContour.RIGHT_EYEBROW_TOP,
        FaceContour.RIGHT_EYEBROW_BOTTOM,
        FaceContour.LEFT_EYE,
        FaceContour.RIGHT_EYE,
        FaceContour.UPPER_LIP_TOP,
        FaceContour.UPPER_LIP_BOTTOM,
        FaceContour.LOWER_LIP_TOP,
        FaceContour.LOWER_LIP_BOTTOM,
        FaceContour.NOSE_BRIDGE,
        FaceContour.NOSE_BOTTOM,
        FaceContour.LEFT_CHEEK,
        FaceContour.RIGHT_CHEEK
      };

    String[] faceContoursTypesStrings = {
        "FACE",
        "LEFT_EYEBROW_TOP",
        "LEFT_EYEBROW_BOTTOM",
        "RIGHT_EYEBROW_TOP",
        "RIGHT_EYEBROW_BOTTOM",
        "LEFT_EYE",
        "RIGHT_EYE",
        "UPPER_LIP_TOP",
        "UPPER_LIP_BOTTOM",
        "LOWER_LIP_TOP",
        "LOWER_LIP_BOTTOM",
        "NOSE_BRIDGE",
        "NOSE_BOTTOM",
        "LEFT_CHEEK",
        "RIGHT_CHEEK"
      };

    WritableMap faceContoursTypesMap = new WritableNativeMap();

      for (int i = 0; i < faceContoursTypesStrings.length; i++) {
        FaceContour contour = face.getContour(faceContoursTypes[i]);
        List<PointF> points = contour.getPoints();
        WritableNativeArray pointsArray = new WritableNativeArray();

          for (int j = 0; j < points.size(); j++) {
            WritableMap currentPointsMap = new WritableNativeMap();

            currentPointsMap.putDouble("x", points.get(j).x);
            currentPointsMap.putDouble("y", points.get(j).y);

            pointsArray.pushMap(currentPointsMap);
          }
          faceContoursTypesMap.putArray(faceContoursTypesStrings[contour.getFaceContourType() - 1], pointsArray);
      }

    return faceContoursTypesMap;
  }


  @Override
  public Object callback(@NonNull Frame frame, @Nullable Map<String, Object> params) {
    @SuppressLint("UnsafeOptInUsageError")
    Image mediaImage = frame.getImage();

    if (mediaImage != null) {
      InputImage image = InputImage.fromMediaImage(mediaImage, Orientation.Companion.fromUnionValue(frame.getOrientation()).toDegrees());

      Task<List<Face>> task = faceDetector.process(image);
      WritableNativeArray array = new WritableNativeArray();
      try {
        List<Face> faces = Tasks.await(task);
        for (Face face : faces) {

          WritableMap map =  new WritableNativeMap();
          WritableMap frameMap =  new WritableNativeMap();
          frameMap.putInt("width", frame.getWidth());
          frameMap.putInt("height", frame.getHeight());
          map.putMap("frame", frameMap);
          map.putDouble("rollAngle", face.getHeadEulerAngleZ());
          map.putDouble("rollAngle", face.getHeadEulerAngleZ()); // Head is rotated to the left rotZ degrees
          map.putDouble("pitchAngle", face.getHeadEulerAngleX()); // Head is rotated to the right rotX degrees
          map.putDouble("yawAngle", face.getHeadEulerAngleY());  // Head is tilted sideways rotY degrees
          map.putDouble("leftEyeOpenProbability", face.getLeftEyeOpenProbability());
          map.putDouble("rightEyeOpenProbability", face.getRightEyeOpenProbability());
          map.putDouble("smilingProbability", face.getSmilingProbability());


          /*WritableMap contours = processFaceContours(face);
          WritableMap bounds = processBoundingBox(face.getBoundingBox());

          map.putMap("bounds", bounds);
          map.putMap("contours", contours);*/

          Log.println(Log.ASSERT, "ScannerFace", String.valueOf(map));
          array.pushMap(map);
          Log.println(Log.ASSERT, "ScannerResultArray", String.valueOf(convertToArrayList(array)));

        }

        return convertToArrayList(array).toString();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }

    return null;
  }

  public String writableNativeArrayToString(WritableNativeArray array) {
    try {
      WritableMap map = new WritableNativeMap();
      map.putArray("tempKey", array);

      JSONObject jsonObject = new JSONObject(map.toString());
      return jsonObject.getJSONArray("tempKey").toString();
    } catch (Exception e) {
      // Handle error
      return null;
    }
  }

  public ArrayList<Object> convertToArrayList(WritableNativeArray nativeArray) {
    ArrayList<Object> list = new ArrayList<>();

    for (int i = 0; i < nativeArray.size(); i++) {
      ReadableType type = nativeArray.getType(i);

      switch (type) {
        case Null:
          list.add(null);
          break;
        case Boolean:
          list.add(nativeArray.getBoolean(i));
          break;
        case Number:
          // Note: This might simplify numbers. You might want to differentiate between integers, doubles, etc.
          list.add(nativeArray.getDouble(i));
          break;
        case String:
          list.add(nativeArray.getString(i));
          break;
        case Map:
          // If you want to convert nested maps to HashMaps, you'll need another conversion function
          list.add(nativeArray.getMap(i));
          break;
        case Array:
          list.add(convertToArrayList((WritableNativeArray) nativeArray.getArray(i)));
          break;
        default:
          throw new IllegalArgumentException("Unknown data type: " + type);
      }
    }

    return list;
  }


  public VisionCameraPluginScanFaces() {}
}

==> vision-camera-plugin-scan-faces-0.2.5.tgz: package/android/src/main/java/com/visioncamerapluginscanfaces/VisionCameraPluginScanFaces.java <==
package com.visioncamerapluginscanfaces;

import static java.lang.Math.ceil;
import android.annotation.SuppressLint;
import android.graphics.PointF;
import android.graphics.Rect;
import android.media.Image;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.WritableNativeArray;
import com.facebook.react.bridge.WritableNativeMap;
import com.google.android.gms.tasks.Task;
import com.google.android.gms.tasks.Tasks;
import com.google.mlkit.vision.common.InputImage;
import com.google.mlkit.vision.face.Face;
import com.google.mlkit.vision.face.FaceContour;
import com.google.mlkit.vision.face.FaceDetection;
import com.google.mlkit.vision.face.FaceDetector;
import com.google.mlkit.vision.face.FaceDetectorOptions;

import com.mrousavy.camera.frameprocessor.Frame;
import com.mrousavy.camera.frameprocessor.FrameProcessorPlugin;
import com.mrousavy.camera.parsers.Orientation;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;


public class VisionCameraPluginScanFaces extends FrameProcessorPlugin {

  FaceDetectorOptions options =
    new FaceDetectorOptions.Builder()
      .setPerformanceMode(FaceDetectorOptions.PERFORMANCE_MODE_FAST)
      .setContourMode(FaceDetectorOptions.CONTOUR_MODE_NONE)
      .setClassificationMode(FaceDetectorOptions.CLASSIFICATION_MODE_ALL)
      .setMinFaceSize(0.15f)
      .build();

  FaceDetector faceDetector = FaceDetection.getClient(options);

  private Map<String, Object> processBoundingBox(Rect boundingBox) {
    Map<String, Object> bounds = new HashMap<>();

    // Calculate offset (we need to center the overlay on the target)
    Double offsetX = (boundingBox.exactCenterX() - Math.ceil(boundingBox.width())) / 2.0f;
    Double offsetY = (boundingBox.exactCenterY() - Math.ceil(boundingBox.height())) / 2.0f;

    Double x = boundingBox.right + offsetX;
    Double y = boundingBox.top + offsetY;

    bounds.put("x",(double) boundingBox.centerX() + (boundingBox.centerX() - x));
    bounds.put("y",(double) boundingBox.centerY() + (y - boundingBox.centerY()));
    bounds.put("width", (double) boundingBox.width());
    bounds.put("height", (double) boundingBox.height());
    bounds.put("boundingCenterX", (double) boundingBox.centerX());
    bounds.put("boundingCenterY", (double) boundingBox.centerY());
    bounds.put("boundingExactCenterX",(double) boundingBox.exactCenterX());
    bounds.put("boundingExactCenterY",(double) boundingBox.exactCenterY());

    return bounds;
  }

  private Map<String, Object> processFaceContours(Face face) {
    // All faceContours
    int[] faceContoursTypes =
      new int[] {
        FaceContour.FACE,
        FaceContour.LEFT_EYEBROW_TOP,
        FaceContour.LEFT_EYEBROW_BOTTOM,
        FaceContour.RIGHT_EYEBROW_TOP,
        FaceContour.RIGHT_EYEBROW_BOTTOM,
        FaceContour.LEFT_EYE,
        FaceContour.RIGHT_EYE,
        FaceContour.UPPER_LIP_TOP,
        FaceContour.UPPER_LIP_BOTTOM,
        FaceContour.LOWER_LIP_TOP,
        FaceContour.LOWER_LIP_BOTTOM,
        FaceContour.NOSE_BRIDGE,
        FaceContour.NOSE_BOTTOM,
        FaceContour.LEFT_CHEEK,
        FaceContour.RIGHT_CHEEK
      };

    String[] faceContoursTypesStrings = {
        "FACE",
        "LEFT_EYEBROW_TOP",
        "LEFT_EYEBROW_BOTTOM",
        "RIGHT_EYEBROW_TOP",
        "RIGHT_EYEBROW_BOTTOM",
        "LEFT_EYE",
        "RIGHT_EYE",
        "UPPER_LIP_TOP",
        "UPPER_LIP_BOTTOM",
        "LOWER_LIP_TOP",
        "LOWER_LIP_BOTTOM",
        "NOSE_BRIDGE",
        "NOSE_BOTTOM",
        "LEFT_CHEEK",
        "RIGHT_CHEEK"
      };

    Map<String, Object> faceContoursTypesMap = new HashMap<>();

    for (int i = 0; i < faceContoursTypesStrings.length; i++) {
      FaceContour contour = face.getContour(faceContoursTypes[i]);
      if(contour != null) {
        List<PointF> points = contour.getPoints();
        List<Map<String, Double>> pointsList = new ArrayList<>();

        for (int j = 0; j < points.size(); j++) {
          Map<String, Double> currentPointsMap = new HashMap<>();
          currentPointsMap.put("x", (double) points.get(j).x);
          currentPointsMap.put("y", (double) points.get(j).y);

          pointsList.add(currentPointsMap);
        }
        faceContoursTypesMap.put(faceContoursTypesStrings[contour.getFaceContourType() - 1], pointsList);
      }
    }

    return faceContoursTypesMap;

  }


  @Override
  public Object callback(@NonNull Frame frame, @Nullable Map<String, Object> params) {
    @SuppressLint("UnsafeOptInUsageError")
    Image mediaImage = frame.getImage();

    if (mediaImage != null) {
      InputImage image = InputImage.fromMediaImage(mediaImage, Orientation.Companion.fromUnionValue(frame.getOrientation()).toDegrees());
      Task<List<Face>> task = faceDetector.process(image);
      List<Map<String, Object>> resultList = new ArrayList<>();

      try {
        List<Face> faces = Tasks.await(task);
        for (Face face : faces) {
          Map<String, Object> map = new HashMap<>();
          Map<String, Object> frameMap = new HashMap<>();
          frameMap.put("width",(double) frame.getWidth());
          frameMap.put("height",(double) frame.getHeight());

          map.put("frame", frameMap);
          map.put("rollAngle",(double) face.getHeadEulerAngleZ());
          map.put("pitchAngle",(double) face.getHeadEulerAngleX());
          map.put("yawAngle",(double) face.getHeadEulerAngleY());
          map.put("leftEyeOpenProbability",(double) face.getLeftEyeOpenProbability());
          map.put("rightEyeOpenProbability",(double) face.getRightEyeOpenProbability());
          map.put("smilingProbability",(double) face.getSmilingProbability());
          map.put("contours", processFaceContours(face));
          map.put("bounds", processBoundingBox(face.getBoundingBox()));

          /*Map<String, Object> contours = processFaceContours(face);
          Map<String, Object> bounds = processBoundingBox(face.getBoundingBox());

          map.put("bounds", bounds);
          map.put("contours", contours);*/

          resultList.add(map);
        }

        Log.d("FaceDetector", resultList.toString());
        return resultList;
      } catch (Exception e) {
        Log.e("FaceDetector", "Error processing face detection", e);
      }
    }

    return null;
  }

  public String writableNativeArrayToString(WritableNativeArray array) {
    try {
      WritableMap map = new WritableNativeMap();
      map.putArray("tempKey", array);

      JSONObject jsonObject = new JSONObject(map.toString());
      return jsonObject.getJSONArray("tempKey").toString();
    } catch (Exception e) {
      // Handle error
      return null;
    }
  }

  public ArrayList<Object> convertToArrayList(WritableNativeArray nativeArray) {
    ArrayList<Object> list = new ArrayList<>();

    for (int i = 0; i < nativeArray.size(); i++) {
      ReadableType type = nativeArray.getType(i);

      switch (type) {
        case Null:
          list.add(null);
          break;
        case Boolean:
          list.add(nativeArray.getBoolean(i));
          break;
        case Number:
          // Note: This might simplify numbers. You might want to differentiate between integers, doubles, etc.
          list.add(nativeArray.getDouble(i));
          break;
        case String:
          list.add(nativeArray.getString(i));
          break;
        case Map:
          // If you want to convert nested maps to HashMaps, you'll need another conversion function
          list.add(nativeArray.getMap(i));
          break;
        case Array:
          list.add(convertToArrayList((WritableNativeArray) nativeArray.getArray(i)));
          break;
        default:
          throw new IllegalArgumentException("Unknown data type: " + type);
      }
    }

    return list;
  }


  public VisionCameraPluginScanFaces() {}
}
